package djmil.cordacheckers;

import java.io.IOException;
import java.security.GeneralSecurityException;

import javax.net.ssl.SSLContext;

import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.client5.http.impl.io.BasicHttpClientConnectionManager;
import org.apache.hc.client5.http.socket.ConnectionSocketFactory;
import org.apache.hc.client5.http.socket.PlainConnectionSocketFactory;
import org.apache.hc.client5.http.ssl.NoopHostnameVerifier;
import org.apache.hc.client5.http.ssl.SSLConnectionSocketFactory;
import org.apache.hc.core5.http.config.Registry;
import org.apache.hc.core5.http.config.RegistryBuilder;
import org.apache.hc.core5.ssl.SSLContexts;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

import djmil.cordacheckers.cordaclient.pojo.holdingIdentity;
import djmil.cordacheckers.cordaclient.pojo.virtualnode;

import static org.assertj.core.api.Assertions.assertThat;


@SpringBootTest
class CordacheckersApplicationTests {

    @Value("${trust.store}")
    private Resource trustStore;

    @Value("${trust.store.password}")
    private String trustStorePassword;

	@Test
	void contextLoads() {
	}

	@Test
	void givenAcceptOnlyCACertificates_whenHttpsUrlIsConsumed_thenOk() throws GeneralSecurityException, IOException {
		final SSLContext sslContext = SSLContexts.custom()
			.loadTrustMaterial(trustStore.getURL(), trustStorePassword.toCharArray())
			.build();
		final SSLConnectionSocketFactory sslsf = new SSLConnectionSocketFactory(sslContext, NoopHostnameVerifier.INSTANCE);
		final Registry<ConnectionSocketFactory> socketFactoryRegistry = RegistryBuilder.<ConnectionSocketFactory> create()
			.register("https", sslsf)
			.register("http", new PlainConnectionSocketFactory())
			.build();

		final BasicHttpClientConnectionManager connectionManager = 
			new BasicHttpClientConnectionManager(socketFactoryRegistry);
		final CloseableHttpClient httpClient = HttpClients.custom()
			.setConnectionManager(connectionManager)
			.build();

		final HttpComponentsClientHttpRequestFactory requestFactory =
			new HttpComponentsClientHttpRequestFactory(httpClient);
		//requestFactory.setReadTimeout(readTimeout);
		//requestFactory.setConnectTimeout(connectTimeout);


		// Request authorization header
		HttpHeaders headers = new HttpHeaders();
		headers.setBasicAuth("admin", "admin");

		// String authStr = "username:password";
		// String base64Creds = Base64.getEncoder().encodeToString(authStr.getBytes());
		// headers.add("Authorization", "Basic " + base64Creds);
		
		// Request
		final HttpEntity<String> request = new HttpEntity<>(headers);

		final ResponseEntity<virtualnode> response = new RestTemplate(requestFactory)
			.exchange("https://localhost:8888/api/v1/virtualnode", HttpMethod.GET, request, virtualnode.class);

		assertThat(response.getStatusCode()).isEqualTo(HttpStatus.OK);
		assertThat(response.hasBody());

		virtualnode vNode = response.getBody();
		assertThat(vNode).isNotNull();
		if (vNode != null) {
			assertThat(vNode.virtualNodes().size() == 5);
		}

		holdingIdentity identity = vNode.virtualNodes().get(0).holdingIdentity();
		assertThat(identity.x500Name().contains("NotaryRep1"));
	}
}
